@@ -1,6 +1,5 @@
 /* PEF support for BFD.
-   Copyright 1999, 2000, 2001, 2002, 2005, 2007
-   Free Software Foundation, Inc.
+   Copyright (C) 1999-2017 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -19,6 +18,7 @@
    Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston,
    MA 02110-1301, USA.  */
 
+#include "sysdep.h"
 #include "bfd.h"
 
 #include <stdio.h>
@@ -79,7 +79,7 @@ enum bfd_pef_imported_library_options
 
 struct bfd_pef_imported_symbol
 {
-  unsigned char class;
+  unsigned char symbol_class;
   unsigned long name;
 };
 typedef struct bfd_pef_imported_symbol bfd_pef_imported_symbol;
