@@ -2,8 +2,7 @@
 
 # debug_msg.sh -- a test case for printing debug info for missing symbols.
 
-# Copyright 2006, 2007, 2008, 2009, 2010, 2011, 2013
-# Free Software Foundation, Inc.
+# Copyright (C) 2006-2017 Free Software Foundation, Inc.
 # Written by Ian Lance Taylor <iant@google.com>.
 
 # This file is part of gold.
@@ -65,8 +64,8 @@ check debug_msg.err "debug_msg.o:debug_msg.cc:badref1: error: undefined referenc
 # file recorded in the objects.
 check debug_msg.err ".*/debug_msg.cc:50: error: undefined reference to 'undef_fn1()'"
 check debug_msg.err ".*/debug_msg.cc:55: error: undefined reference to 'undef_fn2()'"
-check debug_msg.err ".*/debug_msg.cc:43: error: undefined reference to 'undef_fn1()'"
-check debug_msg.err ".*/debug_msg.cc:44: error: undefined reference to 'undef_fn2()'"
+check debug_msg.err ".*/debug_msg.cc:4[356]: error: undefined reference to 'undef_fn1()'"
+check debug_msg.err ".*/debug_msg.cc:4[456]: error: undefined reference to 'undef_fn2()'"
 if test "$DEFAULT_TARGET" != "powerpc"
 then
   check debug_msg.err ".*/debug_msg.cc:.*: error: undefined reference to 'undef_int'"
@@ -75,7 +74,7 @@ fi
 # Check we detected the ODR (One Definition Rule) violation.
 check debug_msg.err ": symbol 'Ordering::operator()(int, int)' defined in multiple places (possible ODR violation):"
 check debug_msg.err "odr_violation1.cc:6"
-check debug_msg.err "odr_violation2.cc:12"
+check debug_msg.err "odr_violation2.cc:1[256]"
 
 # Check we don't have ODR false positives:
 check_missing debug_msg.err "OdrDerived::~OdrDerived()"
@@ -85,11 +84,11 @@ check_missing debug_msg.err "__adjust_heap"
 # still flag it for C++ symbols since those are more likely to be
 # unintentional.
 check_missing debug_msg.err ": symbol 'OverriddenCFunction' defined in multiple places (possible ODR violation):"
-check_missing debug_msg.err "odr_violation1.cc:16"
-check_missing debug_msg.err "odr_violation2.cc:23"
+check_missing debug_msg.err "odr_violation1.cc:1[6-8]"
+check_missing debug_msg.err "odr_violation2.cc:2[3-5]"
 check debug_msg.err ": symbol 'SometimesInlineFunction(int)' defined in multiple places (possible ODR violation):"
-check debug_msg.err "debug_msg.cc:68"
-check debug_msg.err "odr_violation2.cc:27"
+check debug_msg.err "debug_msg.cc:6[89]"
+check debug_msg.err "odr_violation2.cc:2[7-9]"
 
 # Check for the same error messages when using --compressed-debug-sections.
 if test -r debug_msg_cdebug.err
@@ -99,23 +98,23 @@ then
   check debug_msg_cdebug.err "debug_msg_cdebug.o:debug_msg.cc:badref1: error: undefined reference to 'undef_int'"
   check debug_msg_cdebug.err ".*/debug_msg.cc:50: error: undefined reference to 'undef_fn1()'"
   check debug_msg_cdebug.err ".*/debug_msg.cc:55: error: undefined reference to 'undef_fn2()'"
-  check debug_msg_cdebug.err ".*/debug_msg.cc:43: error: undefined reference to 'undef_fn1()'"
-  check debug_msg_cdebug.err ".*/debug_msg.cc:44: error: undefined reference to 'undef_fn2()'"
+  check debug_msg_cdebug.err ".*/debug_msg.cc:4[356]: error: undefined reference to 'undef_fn1()'"
+  check debug_msg_cdebug.err ".*/debug_msg.cc:4[456]: error: undefined reference to 'undef_fn2()'"
   if test "$DEFAULT_TARGET" != "powerpc"
   then
     check debug_msg_cdebug.err ".*/debug_msg.cc:.*: error: undefined reference to 'undef_int'"
   fi
   check debug_msg_cdebug.err ": symbol 'Ordering::operator()(int, int)' defined in multiple places (possible ODR violation):"
   check debug_msg_cdebug.err "odr_violation1.cc:6"
-  check debug_msg_cdebug.err "odr_violation2.cc:12"
+  check debug_msg_cdebug.err "odr_violation2.cc:1[256]"
   check_missing debug_msg_cdebug.err "OdrDerived::~OdrDerived()"
   check_missing debug_msg_cdebug.err "__adjust_heap"
   check_missing debug_msg_cdebug.err ": symbol 'OverriddenCFunction' defined in multiple places (possible ODR violation):"
-  check_missing debug_msg_cdebug.err "odr_violation1.cc:16"
-  check_missing debug_msg_cdebug.err "odr_violation2.cc:23"
+  check_missing debug_msg_cdebug.err "odr_violation1.cc:1[6-8]"
+  check_missing debug_msg_cdebug.err "odr_violation2.cc:2[3-5]"
   check debug_msg_cdebug.err ": symbol 'SometimesInlineFunction(int)' defined in multiple places (possible ODR violation):"
-  check debug_msg_cdebug.err "debug_msg.cc:68"
-  check debug_msg_cdebug.err "odr_violation2.cc:27"
+  check debug_msg_cdebug.err "debug_msg.cc:6[89]"
+  check debug_msg_cdebug.err "odr_violation2.cc:2[7-9]"
 fi
 
 # When linking together .so's, we don't catch the line numbers, but we
@@ -125,15 +124,15 @@ check debug_msg_so.err "debug_msg.so: error: undefined reference to 'undef_fn2()
 check debug_msg_so.err "debug_msg.so: error: undefined reference to 'undef_int'"
 check debug_msg_so.err ": symbol 'Ordering::operator()(int, int)' defined in multiple places (possible ODR violation):"
 check debug_msg_so.err "odr_violation1.cc:6"
-check debug_msg_so.err "odr_violation2.cc:12"
+check debug_msg_so.err "odr_violation2.cc:1[256]"
 check_missing debug_msg_so.err "OdrDerived::~OdrDerived()"
 check_missing debug_msg_so.err "__adjust_heap"
 check_missing debug_msg_so.err ": symbol 'OverriddenCFunction' defined in multiple places (possible ODR violation):"
-check_missing debug_msg_so.err "odr_violation1.cc:16"
-check_missing debug_msg_so.err "odr_violation2.cc:23"
+check_missing debug_msg_so.err "odr_violation1.cc:1[6-8]"
+check_missing debug_msg_so.err "odr_violation2.cc:2[3-5]"
 check debug_msg_so.err ": symbol 'SometimesInlineFunction(int)' defined in multiple places (possible ODR violation):"
-check debug_msg_so.err "debug_msg.cc:68"
-check debug_msg_so.err "odr_violation2.cc:27"
+check debug_msg_so.err "debug_msg.cc:6[89]"
+check debug_msg_so.err "odr_violation2.cc:2[7-9]"
 
 # These messages shouldn't need any debug info to detect:
 check debug_msg_ndebug.err "debug_msg_ndebug.so: error: undefined reference to 'undef_fn1()'"
